//
// Created by UserAlex on 10.10.2024.
//

#include "error-message.h"
#include "interpreter.h"
#include "value.h"

#include <inttypes.h>
#include <stdio.h>
#include <stdarg.h>


void ErrorMessage_err(char* message, Interpreter* interpreter, CallFrame* top_frame)
{
	Token* token = Interpreter_ExpectToken(interpreter, top_frame, top_frame->instruction_pointer + 1);
	printf("Error in line: % " PRIi32 ", column: %" PRIi32, token->context.row, token->context.col);
	printf(": %s\n", message);
}

void ErrorMessage_EmptyStackError(Interpreter* interpreter, CallFrame* top_frame)
{
	ErrorMessage_err("Trying to pop from stack while stack is empty", interpreter, top_frame);

}

void ErrorMessage_StackPushError(Interpreter* interpreter, CallFrame* top_frame)
{
	ErrorMessage_err("Error while trying to push to stack", interpreter, top_frame);
}


void ErrorMessage_IncompatibleTypesError(Interpreter* interpreter, CallFrame* top_frame, Value* expect, Value* got)
{
	// TODO: die nächsten drei Zeilen kommen in den anderen Funktionen auch so ähnlich vor -> da am besten in Funktion auslagern
	size_t total_length = snprintf(NULL, 0, "Incompatible types, expected %s but got %s", Value_ToString(expect->type), Value_ToString(got->type));
	char string[total_length + 1];
	snprintf(string, total_length + 1, "Incompatible types, expected %s but got %s", Value_ToString(expect->type), Value_ToString(got->type));
	ErrorMessage_err( string, interpreter, top_frame);
}

void ErrorMessage_UnknownOperationForTypeError(Interpreter* interpreter, CallFrame* top_frame, char* operation, Value* value)
{
	size_t total_length = snprintf(NULL, 0, "Unknown operation %s for type %s", operation, Value_ToString(value->type));
	char string[total_length + 1];
	snprintf(string, total_length + 1, "Unknown operation %s for type %s", operation, Value_ToString(value->type));
	ErrorMessage_err( string, interpreter, top_frame);
}

void ErrorMessage_OutOfMemoryError(char* context, Interpreter* interpreter, CallFrame* top_frame)
{
	size_t total_length = snprintf(NULL, 0, "Out of memory: %s", context);
	char string[total_length + 1];
	snprintf(string, total_length + 1, "Out of memory: %s", context);
	ErrorMessage_err(string, interpreter, top_frame);
}